import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;
import java.awt.Cursor;


public class MyWorld implements ActionListener {
   public static int WIDTH = 600;  // in pixels
   public static int HEIGHT = 500; // in pixels
   private double t;              // Simulation time
   private Timer pase;
   private double delta_t;        // in seconds
   private double refreshPeriod;  // in seconds
     
   private Vector2D g;
   private float viscosity;
   private PhysicsLabPanel labPanel;
   private Vector<PhysicsElement> elements;
  
   


   public MyWorld(){
      t = 0;
      refreshPeriod = 0.06;      // 80 [ms]
      delta_t = 0.0001;          // 0.1 [ms]
      pase = new Timer((int)(refreshPeriod*1000), this);
      g = new Vector2D(0, 0);
      viscosity = 0.0f;
      elements = new Vector<PhysicsElement>();
      labPanel = null;
       
   }
   public void setPanel (PhysicsLabPanel p) {
      labPanel = p;
   }
   public void setDelta_t(double delta) {
      delta_t = delta;
   }
   public void setRefreshPeriod (double rp) {
      refreshPeriod = rp;
      pase.setDelay((int)(refreshPeriod*1000));
   }
   public void setGravity (Vector2D gravity) {
      g = gravity;
   }
   public void setViscosity(float v){
      viscosity = v;
   }
   public void addElement(PhysicsElement e) {
      elements.add(e);
      labPanel.repaint();
 
 	}
   public void removeElement(PhysicsElement e) {
     if (e == null) return;
    	elements.remove(e);
      labPanel.repaint();
   }
    public void start() { //Este método debe ser modificado
        Vector2D zero = new Vector2D(); //Se inicializa en (0,0)
        if(pase.isRunning()) return;     
	       for (PhysicsElement e: elements){
                   e.setInitialState(g, viscosity);
                   if (e instanceof Ball){// Si el elemento es una Ball, entonces se deben resetear la velocidad y la aceleración
                   ((Ball)e).setSpeed(zero);//Se definio el metodo en la clase Ball
                   ((Ball)e).setAccel(zero);//Idem a setSpeed
                   }
               }             
         
      labPanel.desableMouseListener();
      pase.start();      

   }   
    public void resume() {// El método start ahora es Resume
     if(pase.isRunning()) return;     
	       for (PhysicsElement e: elements)
         e.setInitialState(g, viscosity);
      labPanel.desableMouseListener();
      pase.start();      

   }
   public void stop(){
      pase.stop();
      labPanel.enableMouseListener();
   }
   public void actionPerformed (ActionEvent event) {  // simulate passing time
      double nextStop=t+refreshPeriod;
      for (; t<nextStop; t+=delta_t){
      for (PhysicsElement e: elements)
         e.computeNextState(delta_t, g, viscosity);
      for (PhysicsElement e: elements)
         e.updateState();
      }
      labPanel.repaint();
   }
   public void paintComponent(Graphics2D g2d){
      for (PhysicsElement e: elements)
         if (e instanceof Drawable)
            ((Drawable)e).draw(g2d);
   }
   public Drawable find(int x, int y) {
      for (PhysicsElement e: elements)
         if (e instanceof Drawable){
            Drawable de = (Drawable) e;
            if (de.contains(x,y)) return de;
         }
      return null;
   }  
   public void setCursor(){
	labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
   } 
   public void resetCursor(){
	labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
   }



   public AttachableElement findAttachableElement(int x, int y) {
      for (PhysicsElement e :elements)
         if (e instanceof AttachableElement) {
            AttachableElement ae =(AttachableElement) e;
            if (ae instanceof Drawable)
            if (((Drawable)ae).contains(x,y)) return ae;
         }
      return null;
   }
} 
